@@ -1,5 +1,9 @@
-import re, textwrap
-import sys, os
+import os
+import re
+import struct
+import sys
+import textwrap
+
 sys.path.insert(0, os.path.dirname(__file__))
 import ufunc_docstrings as docstrings
 sys.path.pop(0)
@@ -8,25 +12,40 @@ Zero = "PyUFunc_Zero"
 One = "PyUFunc_One"
 None_ = "PyUFunc_None"
 
+# Sentinel value to specify that the loop for the given TypeDescription uses the
+# pointer to arrays as its func_data.
+UsesArraysAsData = object()
+
+
 class TypeDescription(object):
-    """Type signature for a ufunc
+    """Type signature for a ufunc.
 
     Attributes
     ----------
-
-    type: character representing the type
-    func_data:
-    in_:
-    out:
+    type : str
+        Character representing the nominal type.
+    func_data : str or None or UsesArraysAsData, optional
+        The string representing the expression to insert into the data array, if
+        any.
+    in_ : str or None, optional
+        The typecode(s) of the inputs.
+    out : str or None, optional
+        The typecode(s) of the outputs.
+    astype : dict or None, optional
+        If astype['x'] is 'y', uses PyUFunc_x_x_As_y_y/PyUFunc_xx_x_As_yy_y
+        instead of PyUFunc_x_x/PyUFunc_xx_x.
     """
-    def __init__(self, type, f=None, in_=None, out=None):
+    def __init__(self, type, f=None, in_=None, out=None, astype=None):
         self.type = type
         self.func_data = f
+        if astype is None:
+            astype = {}
+        self.astype_dict = astype
         if in_ is not None:
-            in_ = in_.replace('.', type)
+            in_ = in_.replace('P', type)
         self.in_ = in_
         if out is not None:
-            out = out.replace('.', type)
+            out = out.replace('P', type)
         self.out = out
 
     def finish_signature(self, nin, nout):
@@ -36,8 +55,9 @@ class TypeDescription(object):
         if self.out is None:
             self.out = self.type * nout
         assert len(self.out) == nout
+        self.astype = self.astype_dict.get(self.type, None)
 
-_fdata_map = dict(f='npy_%sf', d='npy_%s', g='npy_%sl',
+_fdata_map = dict(e='npy_%sf', f='npy_%sf', d='npy_%s', g='npy_%sl',
                   F='nc_%sf', D='nc_%s', G='nc_%sl')
 def build_func_data(types, f):
     func_data = []
@@ -46,7 +66,7 @@ def build_func_data(types, f):
         func_data.append(d)
     return func_data
 
-def TD(types, f=None, in_=None, out=None):
+def TD(types, f=None, astype=None, in_=None, out=None):
     if f is not None:
         if isinstance(f, str):
             func_data = build_func_data(types, f)
@@ -65,7 +85,7 @@ def TD(types, f=None, in_=None, out=None):
         out = (None,) * len(types)
     tds = []
     for t, fd, i, o in zip(types, func_data, in_, out):
-        tds.append(TypeDescription(t, f=fd, in_=i, out=o))
+        tds.append(TypeDescription(t, f=fd, in_=i, out=o, astype=astype))
     return tds
 
 class Ufunc(object):
@@ -94,6 +114,43 @@ class Ufunc(object):
         for td in self.type_descriptions:
             td.finish_signature(self.nin, self.nout)
 
+# String-handling utilities to avoid locale-dependence.
+
+import string
+if sys.version_info[0] < 3:
+    UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+else:
+    UPPER_TABLE = bytes.maketrans(bytes(string.ascii_lowercase, "ascii"), 
+            bytes(string.ascii_uppercase, "ascii"))
+
+def english_upper(s):
+    """ Apply English case rules to convert ASCII strings to all upper case.
+
+    This is an internal utility function to replace calls to str.upper() such
+    that we can avoid changing behavior with changing locales. In particular,
+    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
+    both lowercase and uppercase. Thus, "i".upper() != "I" in a "tr" locale.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    uppered : str
+
+    Examples
+    --------
+    >>> from numpy.lib.utils import english_upper
+    >>> english_upper('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
+    'ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
+    >>> english_upper('')
+    ''
+    """
+    uppered = s.translate(UPPER_TABLE)
+    return uppered
+
+
 #each entry in defdict is a Ufunc object.
 
 #name: [string of chars for which it is defined,
@@ -116,61 +173,87 @@ chartoname = {'?': 'bool',
               'L': 'ulong',
               'q': 'longlong',
               'Q': 'ulonglong',
+              'e': 'half',
               'f': 'float',
               'd': 'double',
               'g': 'longdouble',
               'F': 'cfloat',
               'D': 'cdouble',
               'G': 'clongdouble',
+              'M': 'datetime',
+              'm': 'timedelta',
               'O': 'OBJECT',
-              # M is like O, but calls a method of the object instead
+              # '.' is like 'O', but calls a method of the object instead
               # of a function
-              'M': 'OBJECT',
+              'P': 'OBJECT',
               }
 
-all = '?bBhHiIlLqQfdgFDGO'
+all = '?bBhHiIlLqQefdgFDGOMm'
 O = 'O'
-M = 'M'
+P = 'P'
 ints = 'bBhHiIlLqQ'
+times = 'Mm'
 intsO = ints + O
 bints = '?' + ints
 bintsO = bints + O
-flts = 'fdg'
+flts = 'efdg'
 fltsO = flts + O
-fltsM = flts + M
+fltsP = flts + P
 cmplx = 'FDG'
 cmplxO = cmplx + O
-cmplxM = cmplx + M
+cmplxP = cmplx + P
 inexact = flts + cmplx
 noint = inexact+O
-nointM = inexact+M
-allM = bints+flts+cmplxM
+nointP = inexact+P
+allP = bints+times+flts+cmplxP
 nobool = all[1:]
-nobool_or_obj = all[1:-1]
+noobj = all[:-3]+all[-2:]
+nobool_or_obj = all[1:-3]+all[-2:]
 intflt = ints+flts
-intfltcmplx = nobool_or_obj
-nocmplx = bints+flts
+intfltcmplx = ints+flts+cmplx
+nocmplx = bints+times+flts
 nocmplxO = nocmplx+O
-nocmplxM = nocmplx+M
-noobj = all[:-1]
+nocmplxP = nocmplx+P
+notimes_or_obj = bints + inexact
 
+# Find which code corresponds to int64.
+int64 = ''
+uint64 = ''
+for code in 'bhilq':
+    if struct.calcsize(code) == 8:
+        int64 = code
+        uint64 = english_upper(code)
+        break
+
+# This dictionary describes all the ufunc implementations, generating
+# all the function names and their corresponding ufunc signatures.  TD is
+# an object which expands a list of character codes into an array of
+# TypeDescriptions.
 defdict = {
 'add' :
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.add'),
-          TD(noobj),
+          TD(notimes_or_obj),
+          [TypeDescription('M', UsesArraysAsData, 'Mm', 'M'),
+           TypeDescription('m', UsesArraysAsData, 'mm', 'm'),
+           TypeDescription('M', UsesArraysAsData, 'mM', 'M'),
+          ],
           TD(O, f='PyNumber_Add'),
           ),
 'subtract' :
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.subtract'),
-          TD(noobj),
+          TD(notimes_or_obj),
+          [TypeDescription('M', UsesArraysAsData, 'Mm', 'M'),
+           TypeDescription('m', UsesArraysAsData, 'mm', 'm'),
+           TypeDescription('M', UsesArraysAsData, 'MM', 'm'),
+          ],
           TD(O, f='PyNumber_Subtract'),
           ),
 'multiply' :
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.multiply'),
-          TD(noobj),
+          TD(notimes_or_obj),
           TD(O, f='PyNumber_Multiply'),
           ),
 'divide' :
@@ -188,7 +271,7 @@ defdict = {
 'true_divide' :
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.true_divide'),
-          TD('bBhH', out='f'),
+          TD('bBhH', out='d'),
           TD('iIlLqQ', out='d'),
           TD(flts+cmplx),
           TD(O, f='PyNumber_TrueDivide'),
@@ -196,26 +279,26 @@ defdict = {
 'conjugate' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.conjugate'),
-          TD(nobool_or_obj),
-          TD(M, f='conjugate'),
+          TD(ints+flts+cmplx),
+          TD(P, f='conjugate'),
           ),
 'fmod' :
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.fmod'),
           TD(ints),
-          TD(flts, f='fmod'),
-          TD(M, f='fmod'),
+          TD(flts, f='fmod', astype={'e':'f'}),
+          TD(P, f='fmod'),
           ),
 'square' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.square'),
-          TD(nobool_or_obj),
+          TD(ints+inexact),
           TD(O, f='Py_square'),
           ),
 'reciprocal' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.reciprocal'),
-          TD(nobool_or_obj),
+          TD(ints+inexact),
           TD(O, f='Py_reciprocal'),
           ),
 'ones_like' :
@@ -228,20 +311,25 @@ defdict = {
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.power'),
           TD(ints),
-          TD(inexact, f='pow'),
+          TD(inexact, f='pow', astype={'e':'f'}),
           TD(O, f='npy_ObjectPower'),
           ),
 'absolute' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.absolute'),
-          TD(nocmplx),
+          TD(bints+flts+times),
           TD(cmplx, out=('f', 'd', 'g')),
           TD(O, f='PyNumber_Absolute'),
           ),
+'_arg' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath._arg'),
+          TD(cmplx, out=('f', 'd', 'g')),
+          ),
 'negative' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.negative'),
-          TD(nocmplx),
+          TD(bints+flts+times),
           TD(cmplx, f='neg'),
           TD(O, f='PyNumber_Negative'),
           ),
@@ -284,25 +372,25 @@ defdict = {
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.logical_and'),
           TD(noobj, out='?'),
-          TD(M, f='logical_and'),
+          TD(P, f='logical_and'),
           ),
 'logical_not' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.logical_not'),
           TD(noobj, out='?'),
-          TD(M, f='logical_not'),
+          TD(P, f='logical_not'),
           ),
 'logical_or' :
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.logical_or'),
           TD(noobj, out='?'),
-          TD(M, f='logical_or'),
+          TD(P, f='logical_or'),
           ),
 'logical_xor' :
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logical_xor'),
           TD(noobj, out='?'),
-          TD(M, f='logical_xor'),
+          TD(P, f='logical_xor'),
           ),
 'maximum' :
     Ufunc(2, 1, None,
@@ -331,13 +419,14 @@ defdict = {
 'logaddexp' :
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logaddexp'),
-          TD(flts, f="logaddexp")
+          TD(flts, f="logaddexp", astype={'e':'f'})
           ),
 'logaddexp2' :
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.logaddexp2'),
-          TD(flts, f="logaddexp2")
+          TD(flts, f="logaddexp2", astype={'e':'f'})
           ),
+# FIXME: decide if the times should have the bitwise operations.
 'bitwise_and' :
     Ufunc(2, 1, One,
           docstrings.get('numpy.core.umath.bitwise_and'),
@@ -377,178 +466,178 @@ defdict = {
 'degrees' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.degrees'),
-          TD(fltsM, f='degrees'),
+          TD(fltsP, f='degrees', astype={'e':'f'}),
           ),
 'rad2deg' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.rad2deg'),
-          TD(fltsM, f='rad2deg'),
+          TD(fltsP, f='rad2deg', astype={'e':'f'}),
           ),
 'radians' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.radians'),
-          TD(fltsM, f='radians'),
+          TD(fltsP, f='radians', astype={'e':'f'}),
           ),
 'deg2rad' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.deg2rad'),
-          TD(fltsM, f='deg2rad'),
+          TD(fltsP, f='deg2rad', astype={'e':'f'}),
           ),
 'arccos' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arccos'),
-          TD(inexact, f='acos'),
-          TD(M, f='arccos'),
+          TD(inexact, f='acos', astype={'e':'f'}),
+          TD(P, f='arccos'),
           ),
 'arccosh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arccosh'),
-          TD(inexact, f='acosh'),
-          TD(M, f='arccosh'),
+          TD(inexact, f='acosh', astype={'e':'f'}),
+          TD(P, f='arccosh'),
           ),
 'arcsin' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arcsin'),
-          TD(inexact, f='asin'),
-          TD(M, f='arcsin'),
+          TD(inexact, f='asin', astype={'e':'f'}),
+          TD(P, f='arcsin'),
           ),
 'arcsinh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arcsinh'),
-          TD(inexact, f='asinh'),
-          TD(M, f='arcsinh'),
+          TD(inexact, f='asinh', astype={'e':'f'}),
+          TD(P, f='arcsinh'),
           ),
 'arctan' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arctan'),
-          TD(inexact, f='atan'),
-          TD(M, f='arctan'),
+          TD(inexact, f='atan', astype={'e':'f'}),
+          TD(P, f='arctan'),
           ),
 'arctanh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.arctanh'),
-          TD(inexact, f='atanh'),
-          TD(M, f='arctanh'),
+          TD(inexact, f='atanh', astype={'e':'f'}),
+          TD(P, f='arctanh'),
           ),
 'cos' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.cos'),
-          TD(inexact, f='cos'),
-          TD(M, f='cos'),
+          TD(inexact, f='cos', astype={'e':'f'}),
+          TD(P, f='cos'),
           ),
 'sin' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sin'),
-          TD(inexact, f='sin'),
-          TD(M, f='sin'),
+          TD(inexact, f='sin', astype={'e':'f'}),
+          TD(P, f='sin'),
           ),
 'tan' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.tan'),
-          TD(inexact, f='tan'),
-          TD(M, f='tan'),
+          TD(inexact, f='tan', astype={'e':'f'}),
+          TD(P, f='tan'),
           ),
 'cosh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.cosh'),
-          TD(inexact, f='cosh'),
-          TD(M, f='cosh'),
+          TD(inexact, f='cosh', astype={'e':'f'}),
+          TD(P, f='cosh'),
           ),
 'sinh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sinh'),
-          TD(inexact, f='sinh'),
-          TD(M, f='sinh'),
+          TD(inexact, f='sinh', astype={'e':'f'}),
+          TD(P, f='sinh'),
           ),
 'tanh' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.tanh'),
-          TD(inexact, f='tanh'),
-          TD(M, f='tanh'),
+          TD(inexact, f='tanh', astype={'e':'f'}),
+          TD(P, f='tanh'),
           ),
 'exp' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.exp'),
-          TD(inexact, f='exp'),
-          TD(M, f='exp'),
+          TD(inexact, f='exp', astype={'e':'f'}),
+          TD(P, f='exp'),
           ),
 'exp2' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.exp2'),
-          TD(inexact, f='exp2'),
-          TD(M, f='exp2'),
+          TD(inexact, f='exp2', astype={'e':'f'}),
+          TD(P, f='exp2'),
           ),
 'expm1' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.expm1'),
-          TD(inexact, f='expm1'),
-          TD(M, f='expm1'),
+          TD(inexact, f='expm1', astype={'e':'f'}),
+          TD(P, f='expm1'),
           ),
 'log' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log'),
-          TD(inexact, f='log'),
-          TD(M, f='log'),
+          TD(inexact, f='log', astype={'e':'f'}),
+          TD(P, f='log'),
           ),
 'log2' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log2'),
-          TD(inexact, f='log2'),
-          TD(M, f='log2'),
+          TD(inexact, f='log2', astype={'e':'f'}),
+          TD(P, f='log2'),
           ),
 'log10' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log10'),
-          TD(inexact, f='log10'),
-          TD(M, f='log10'),
+          TD(inexact, f='log10', astype={'e':'f'}),
+          TD(P, f='log10'),
           ),
 'log1p' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.log1p'),
-          TD(inexact, f='log1p'),
-          TD(M, f='log1p'),
+          TD(inexact, f='log1p', astype={'e':'f'}),
+          TD(P, f='log1p'),
           ),
 'sqrt' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.sqrt'),
-          TD(inexact, f='sqrt'),
-          TD(M, f='sqrt'),
+          TD(inexact, f='sqrt', astype={'e':'f'}),
+          TD(P, f='sqrt'),
           ),
 'ceil' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.ceil'),
-          TD(flts, f='ceil'),
-          TD(M, f='ceil'),
+          TD(flts, f='ceil', astype={'e':'f'}),
+          TD(P, f='ceil'),
           ),
 'trunc' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.trunc'),
-          TD(flts, f='trunc'),
-          TD(M, f='trunc'),
+          TD(flts, f='trunc', astype={'e':'f'}),
+          TD(P, f='trunc'),
           ),
 'fabs' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.fabs'),
-          TD(flts, f='fabs'),
-          TD(M, f='fabs'),
+          TD(flts, f='fabs', astype={'e':'f'}),
+          TD(P, f='fabs'),
        ),
 'floor' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.floor'),
-          TD(flts, f='floor'),
-          TD(M, f='floor'),
+          TD(flts, f='floor', astype={'e':'f'}),
+          TD(P, f='floor'),
           ),
 'rint' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.rint'),
-          TD(inexact, f='rint'),
-          TD(M, f='rint'),
+          TD(inexact, f='rint', astype={'e':'f'}),
+          TD(P, f='rint'),
           ),
 'arctan2' :
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.arctan2'),
-          TD(flts, f='atan2'),
-          TD(M, f='arctan2'),
+          TD(flts, f='atan2', astype={'e':'f'}),
+          TD(P, f='arctan2'),
           ),
 'remainder' :
     Ufunc(2, 1, None,
@@ -559,8 +648,8 @@ defdict = {
 'hypot' :
     Ufunc(2, 1, None,
           docstrings.get('numpy.core.umath.hypot'),
-          TD(flts, f='hypot'),
-          TD(M, f='hypot'),
+          TD(flts, f='hypot', astype={'e':'f'}),
+          TD(P, f='hypot'),
           ),
 'isnan' :
     Ufunc(1, 1, None,
@@ -582,6 +671,21 @@ defdict = {
           docstrings.get('numpy.core.umath.signbit'),
           TD(flts, out='?'),
           ),
+'copysign' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.copysign'),
+          TD(flts),
+          ),
+'nextafter' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.nextafter'),
+          TD(flts),
+          ),
+'spacing' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.spacing'),
+          TD(flts),
+          ),
 'modf' :
     Ufunc(1, 2, None,
           docstrings.get('numpy.core.umath.modf'),
@@ -589,6 +693,10 @@ defdict = {
           ),
 }
 
+if sys.version_info[0] >= 3:
+    # Will be aliased to true_divide in umathmodule.c.src:InitOtherOperators
+    del defdict['divide']
+
 def indent(st,spaces):
     indention = ' '*spaces
     indented = indention + st.replace('\n','\n'+indention)
@@ -596,68 +704,37 @@ def indent(st,spaces):
     indented = re.sub(r' +$',r'',indented)
     return indented
 
-chartotype1 = {'f': 'f_f',
+chartotype1 = {'e': 'e_e',
+               'f': 'f_f',
                'd': 'd_d',
                'g': 'g_g',
                'F': 'F_F',
                'D': 'D_D',
                'G': 'G_G',
                'O': 'O_O',
-               'M': 'O_O_method'}
+               'P': 'O_O_method'}
 
-chartotype2 = {'f': 'ff_f',
+chartotype2 = {'e': 'ee_e',
+               'f': 'ff_f',
                'd': 'dd_d',
                'g': 'gg_g',
                'F': 'FF_F',
                'D': 'DD_D',
                'G': 'GG_G',
                'O': 'OO_O',
-               'M': 'OO_O_method'}
+               'P': 'OO_O_method'}
 #for each name
 # 1) create functions, data, and signature
 # 2) fill in functions and data in InitOperators
 # 3) add function.
 
-# String-handling utilities to avoid locale-dependence.
-
-import string
-UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
-
-def english_upper(s):
-    """ Apply English case rules to convert ASCII strings to all upper case.
-
-    This is an internal utility function to replace calls to str.upper() such
-    that we can avoid changing behavior with changing locales. In particular,
-    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
-    both lowercase and uppercase. Thus, "i".upper() != "I" in a "tr" locale.
-
-    Parameters
-    ----------
-    s : str
-
-    Returns
-    -------
-    uppered : str
-
-    Examples
-    --------
-    >>> from numpy.lib.utils import english_upper
-    >>> english_upper('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
-    'ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
-    >>> english_upper('')
-    ''
-    """
-    uppered = s.translate(UPPER_TABLE)
-    return uppered
-
-
 def make_arrays(funcdict):
     # functions array contains an entry for every type implemented
     #   NULL should be placed where PyUfunc_ style function will be filled in later
     #
     code1list = []
     code2list = []
-    names = funcdict.keys()
+    names = list(funcdict.keys())
     names.sort()
     for name in names:
         uf = funcdict[name]
@@ -674,17 +751,20 @@ def make_arrays(funcdict):
             thedict = chartotype1  # one input and one output
 
         for t in uf.type_descriptions:
-            if t.func_data is not None:
+            if t.func_data not in (None, UsesArraysAsData):
                 funclist.append('NULL')
-                astr = '%s_functions[%d] = PyUFunc_%s;' % \
-                       (name, k, thedict[t.type])
+                astype = ''
+                if not t.astype is None:
+                    astype = '_As_%s' % thedict[t.astype]
+                astr = '%s_functions[%d] = PyUFunc_%s%s;' % \
+                       (name, k, thedict[t.type], astype)
                 code2list.append(astr)
                 if t.type == 'O':
                     astr = '%s_data[%d] = (void *) %s;' % \
                            (name, k, t.func_data)
                     code2list.append(astr)
                     datalist.append('(void *)NULL')
-                elif t.type == 'M':
+                elif t.type == 'P':
                     datalist.append('(void *)"%s"' % t.func_data)
                 else:
                     astr = '%s_data[%d] = (void *) %s;' % \
@@ -693,8 +773,13 @@ def make_arrays(funcdict):
                     datalist.append('(void *)NULL')
                     #datalist.append('(void *)%s' % t.func_data)
                 sub += 1
+            elif t.func_data is UsesArraysAsData:
+                tname = english_upper(chartoname[t.type])
+                datalist.append('(void *)NULL')
+                funclist.append('%s_%s_%s_%s' % (tname, t.in_, t.out, name))
+                code2list.append('PyUFunc_SetUsesArraysAsData(%s_data, %s);' % (name, k))
             else:
-                datalist.append('(void *)NULL');
+                datalist.append('(void *)NULL')
                 tname = english_upper(chartoname[t.type])
                 funclist.append('%s_%s' % (tname, name))
 
@@ -716,13 +801,21 @@ def make_arrays(funcdict):
 
 def make_ufuncs(funcdict):
     code3list = []
-    names = funcdict.keys()
+    names = list(funcdict.keys())
     names.sort()
     for name in names:
         uf = funcdict[name]
         mlist = []
         docstring = textwrap.dedent(uf.docstring).strip()
-        docstring = docstring.encode('string-escape').replace(r'"', r'\"')
+        if sys.version_info[0] < 3:
+            docstring = docstring.encode('string-escape')
+            docstring = docstring.replace(r'"', r'\"')
+        else:
+            docstring = docstring.encode('unicode-escape').decode('ascii')
+            docstring = docstring.replace(r'"', r'\"')
+            # XXX: I don't understand why the following replace is not
+            # necessary in the python 2 case.
+            docstring = docstring.replace(r"'", r"\'")
         # Split the docstring because some compilers (like MS) do not like big
         # string literal in C code. We split at endlines because textwrap.wrap
         # do not play well with \n
