@@ -6,6 +6,9 @@ from numpy.testing import *
 from numpy.core import *
 from numpy.core.multiarray_tests import test_neighborhood_iterator, test_neighborhood_iterator_oob
 
+# Need to test an object that does not fully implement math interface
+from datetime import timedelta
+
 from numpy.compat import asbytes, getexception, strchar
 
 from test_print import in_foreign_locale
@@ -112,6 +115,30 @@ class TestAttributes(TestCase):
         x.fill(x[0])
         assert_equal(x['f1'][1], x['f1'][0])
 
+class TestAssignment(TestCase):
+    def test_assignment_broadcasting(self):
+        a = np.arange(6).reshape(2,3)
+
+        # Broadcasting the input to the output
+        a[...] = np.arange(3)
+        assert_equal(a, [[0,1,2],[0,1,2]])
+        a[...] = np.arange(2).reshape(2,1)
+        assert_equal(a, [[0,0,0],[1,1,1]])
+
+        # For compatibility with <= 1.5, a limited version of broadcasting
+        # the output to the input.
+        #
+        # This behavior is inconsistent with NumPy broadcasting
+        # in general, because it only uses one of the two broadcasting
+        # rules (adding a new "1" dimension to the left of the shape),
+        # applied to the output instead of an input. In NumPy 2.0, this kind
+        # of broadcasting assignment will likely be disallowed.
+        a[...] = np.arange(6)[::-1].reshape(1,2,3)
+        assert_equal(a, [[5,4,3],[2,1,0]])
+        # The other type of broadcasting would require a reduction operation.
+        def assign(a,b):
+            a[...] = b
+        assert_raises(ValueError, assign, a, np.arange(12).reshape(2,2,3))
 
 class TestDtypedescr(TestCase):
     def test_construction(self):
@@ -239,6 +266,53 @@ class TestScalarIndexing(TestCase):
         self.assertRaises(IndexError, subscript, a, (newaxis, 0))
         self.assertRaises(IndexError, subscript, a, (newaxis,)*50)
 
+    def test_overlapping_assignment(self):
+        # With positive strides
+        a = np.arange(4)
+        a[:-1] = a[1:]
+        assert_equal(a, [1,2,3,3])
+
+        a = np.arange(4)
+        a[1:] = a[:-1]
+        assert_equal(a, [0,0,1,2])
+
+        # With positive and negative strides
+        a = np.arange(4)
+        a[:] = a[::-1]
+        assert_equal(a, [3,2,1,0])
+
+        a = np.arange(6).reshape(2,3)
+        a[::-1,:] = a[:,::-1]
+        assert_equal(a, [[5,4,3],[2,1,0]])
+
+        a = np.arange(6).reshape(2,3)
+        a[::-1,::-1] = a[:,::-1]
+        assert_equal(a, [[3,4,5],[0,1,2]])
+
+        # With just one element overlapping
+        a = np.arange(5)
+        a[:3] = a[2:]
+        assert_equal(a, [2,3,4,3,4])
+
+        a = np.arange(5)
+        a[2:] = a[:3]
+        assert_equal(a, [0,1,0,1,2])
+
+        a = np.arange(5)
+        a[2::-1] = a[2:]
+        assert_equal(a, [4,3,2,3,4])
+
+        a = np.arange(5)
+        a[2:] = a[2::-1]
+        assert_equal(a, [0,1,2,1,0])
+
+        a = np.arange(5)
+        a[2::-1] = a[:1:-1]
+        assert_equal(a, [2,3,4,3,4])
+
+        a = np.arange(5)
+        a[:1:-1] = a[2::-1]
+        assert_equal(a, [0,1,0,1,2])
 
 class TestCreation(TestCase):
     def test_from_attribute(self):
@@ -255,6 +329,103 @@ class TestCreation(TestCase):
             msg = 'String conversion for %s' % type
             assert_equal(array(nstr, dtype=type), result, err_msg=msg)
 
+    def test_non_sequence_sequence(self):
+        """Should not segfault.
+
+        Class Fail breaks the sequence protocol for new style classes, i.e.,
+        those derived from object. Class Map is a mapping type indicated by
+        raising a ValueError. At some point we may raise a warning instead
+        of an error in the Fail case.
+
+        """
+        class Fail(object):
+            def __len__(self):
+                return 1
+
+            def __getitem__(self, index):
+                raise ValueError()
+
+        class Map(object):
+            def __len__(self):
+                return 1
+
+            def __getitem__(self, index):
+                raise KeyError()
+
+        a = np.array([Map()])
+        assert_(a.shape == (1,))
+        assert_(a.dtype == np.dtype(object))
+        assert_raises(ValueError, np.array, [Fail()])
+
+
+class TestStructured(TestCase):
+    def test_subarray_field_access(self):
+        a = np.zeros((3, 5), dtype=[('a', ('i4', (2, 2)))])
+        a['a'] = np.arange(60).reshape(3, 5, 2, 2)
+
+        # Since the subarray is always in C-order, these aren't equal
+        assert_(np.any(a['a'].T != a.T['a']))
+
+        # In Fortran order, the subarray gets appended
+        # like in all other cases, not prepended as a special case
+        b = a.copy(order='F')
+        assert_equal(a['a'].shape, b['a'].shape)
+        assert_equal(a.T['a'].shape, a.T.copy()['a'].shape)
+
+
+    def test_subarray_comparison(self):
+        # Check that comparisons between record arrays with
+        # multi-dimensional field types work properly
+        a = np.rec.fromrecords(
+            [([1,2,3],'a', [[1,2],[3,4]]),([3,3,3],'b',[[0,0],[0,0]])],
+            dtype=[('a', ('f4',3)), ('b', np.object), ('c', ('i4',(2,2)))])
+        b = a.copy()
+        assert_equal(a==b, [True,True])
+        assert_equal(a!=b, [False,False])
+        b[1].b = 'c'
+        assert_equal(a==b, [True,False])
+        assert_equal(a!=b, [False,True])
+        for i in range(3):
+            b[0].a = a[0].a
+            b[0].a[i] = 5
+            assert_equal(a==b, [False,False])
+            assert_equal(a!=b, [True,True])
+        for i in range(2):
+            for j in range(2):
+                b = a.copy()
+                b[0].c[i,j] = 10
+                assert_equal(a==b, [False,True])
+                assert_equal(a!=b, [True,False])
+
+        # Check that broadcasting with a subarray works
+        a = np.array([[(0,)],[(1,)]],dtype=[('a','f8')])
+        b = np.array([(0,),(0,),(1,)],dtype=[('a','f8')])
+        assert_equal(a==b, [[True, True, False], [False, False, True]])
+        assert_equal(b==a, [[True, True, False], [False, False, True]])
+        a = np.array([[(0,)],[(1,)]],dtype=[('a','f8',(1,))])
+        b = np.array([(0,),(0,),(1,)],dtype=[('a','f8',(1,))])
+        assert_equal(a==b, [[True, True, False], [False, False, True]])
+        assert_equal(b==a, [[True, True, False], [False, False, True]])
+        a = np.array([[([0,0],)],[([1,1],)]],dtype=[('a','f8',(2,))])
+        b = np.array([([0,0],),([0,1],),([1,1],)],dtype=[('a','f8',(2,))])
+        assert_equal(a==b, [[True, False, False], [False, False, True]])
+        assert_equal(b==a, [[True, False, False], [False, False, True]])
+
+        # Check that broadcasting Fortran-style arrays with a subarray work
+        a = np.array([[([0,0],)],[([1,1],)]],dtype=[('a','f8',(2,))], order='F')
+        b = np.array([([0,0],),([0,1],),([1,1],)],dtype=[('a','f8',(2,))])
+        assert_equal(a==b, [[True, False, False], [False, False, True]])
+        assert_equal(b==a, [[True, False, False], [False, False, True]])
+
+        # Check that incompatible sub-array shapes don't result to broadcasting
+        x = np.zeros((1,), dtype=[('a', ('f4', (1,2))), ('b', 'i1')])
+        y = np.zeros((1,), dtype=[('a', ('f4', (2,))), ('b', 'i1')])
+        assert_equal(x == y, False)
+
+        x = np.zeros((1,), dtype=[('a', ('f4', (2,1))), ('b', 'i1')])
+        y = np.zeros((1,), dtype=[('a', ('f4', (2,))), ('b', 'i1')])
+        assert_equal(x == y, False)
+
 
 class TestBool(TestCase):
     def test_test_interning(self):
@@ -526,6 +697,39 @@ class TestMethods(TestCase):
         msg = "Test complex searchsorted with nans, side='r'"
         b = a.searchsorted(a, side='r')
         assert_equal(b, np.arange(1,10), msg)
+        msg = "Test searchsorted with little endian, side='l'"
+        a = np.array([0,128],dtype='<i4')
+        b = a.searchsorted(np.array(128,dtype='<i4'))
+        assert_equal(b, 1, msg)
+        msg = "Test searchsorted with big endian, side='l'"
+        a = np.array([0,128],dtype='>i4')
+        b = a.searchsorted(np.array(128,dtype='>i4'))
+        assert_equal(b, 1, msg)
+
+    def test_searchsorted_unicode(self):
+        # Test searchsorted on unicode strings.
+
+        # 1.6.1 contained a string length miscalculation in
+        # arraytypes.c.src:UNICODE_compare() which manifested as
+        # incorrect/inconsistent results from searchsorted.
+        a = np.array([u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100185_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100186_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100187_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100189_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100190_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100191_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100192_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100193_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100194_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100195_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100196_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100197_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100198_1',
+                      u'P:\\20x_dapi_cy3\\20x_dapi_cy3_20100199_1'])
+        assert_equal([a.searchsorted(v, 'left') for v in a], np.arange(len(a)))
+        assert_equal([a.searchsorted(v, 'right') for v in a], np.arange(len(a)) + 1)
+        assert_equal([a.searchsorted(a[i], 'left') for i in range(len(a))], np.arange(len(a)))
+        assert_equal([a.searchsorted(a[i], 'right') for i in range(len(a))], np.arange(len(a)) + 1)
 
     def test_flatten(self):
         x0 = np.array([[1,2,3],[4,5,6]], np.int32)
@@ -549,6 +753,55 @@ class TestMethods(TestCase):
         assert_equal(np.dot(a, b), a.dot(b))
         assert_equal(np.dot(np.dot(a, b), c), a.dot(b).dot(c))
 
+    def test_ravel(self):
+        a = np.array([[0,1],[2,3]])
+        assert_equal(a.ravel(), [0,1,2,3])
+        assert_(not a.ravel().flags.owndata)
+        assert_equal(a.ravel('F'), [0,2,1,3])
+        assert_equal(a.ravel(order='C'), [0,1,2,3])
+        assert_equal(a.ravel(order='F'), [0,2,1,3])
+        assert_equal(a.ravel(order='A'), [0,1,2,3])
+        assert_(not a.ravel(order='A').flags.owndata)
+        assert_equal(a.ravel(order='K'), [0,1,2,3])
+        assert_(not a.ravel(order='K').flags.owndata)
+        assert_equal(a.ravel(), a.reshape(-1))
+
+        a = np.array([[0,1],[2,3]], order='F')
+        assert_equal(a.ravel(), [0,1,2,3])
+        assert_equal(a.ravel(order='A'), [0,2,1,3])
+        assert_equal(a.ravel(order='K'), [0,2,1,3])
+        assert_(not a.ravel(order='A').flags.owndata)
+        assert_(not a.ravel(order='K').flags.owndata)
+        assert_equal(a.ravel(), a.reshape(-1))
+        assert_equal(a.ravel(order='A'), a.reshape(-1, order='A'))
+
+        a = np.array([[0,1],[2,3]])[::-1,:]
+        assert_equal(a.ravel(), [2,3,0,1])
+        assert_equal(a.ravel(order='C'), [2,3,0,1])
+        assert_equal(a.ravel(order='F'), [2,0,3,1])
+        assert_equal(a.ravel(order='A'), [2,3,0,1])
+        # 'K' doesn't reverse the axes of negative strides
+        assert_equal(a.ravel(order='K'), [2,3,0,1])
+        assert_(a.ravel(order='K').flags.owndata)
+
+    def test_setasflat(self):
+        # In this case, setasflat can treat a as a flat array,
+        # and must treat b in chunks of 3
+        a = np.arange(3*3*4).reshape(3,3,4)
+        b = np.arange(3*4*3, dtype='f4').reshape(3,4,3).T
+
+        assert_(not np.all(a.ravel() == b.ravel()))
+        a.setasflat(b)
+        assert_equal(a.ravel(), b.ravel())
+
+        # A case where the strides of neither a nor b can be collapsed
+        a = np.arange(3*2*4).reshape(3,2,4)[:,:,:-1]
+        b = np.arange(3*3*3, dtype='f4').reshape(3,3,3).T[:,:,:-1]
+
+        assert_(not np.all(a.ravel() == b.ravel()))
+        a.setasflat(b)
+        assert_equal(a.ravel(), b.ravel())
+
 class TestSubscripting(TestCase):
     def test_test_zero_rank(self):
         x = array([1,2,3])
@@ -617,6 +870,12 @@ class TestPickling(TestCase):
         p = self._loads(asbytes(s))
         assert_equal(a, p)
 
+    def test_subarray_int_shape(self):
+        s = "cnumpy.core.multiarray\n_reconstruct\np0\n(cnumpy\nndarray\np1\n(I0\ntp2\nS'b'\np3\ntp4\nRp5\n(I1\n(I1\ntp6\ncnumpy\ndtype\np7\n(S'V6'\np8\nI0\nI1\ntp9\nRp10\n(I3\nS'|'\np11\nN(S'a'\np12\ng3\ntp13\n(dp14\ng12\n(g7\n(S'V4'\np15\nI0\nI1\ntp16\nRp17\n(I3\nS'|'\np18\n(g7\n(S'i1'\np19\nI0\nI1\ntp20\nRp21\n(I3\nS'|'\np22\nNNNI-1\nI-1\nI0\ntp23\nb(I2\nI2\ntp24\ntp25\nNNI4\nI1\nI0\ntp26\nbI0\ntp27\nsg3\n(g7\n(S'V2'\np28\nI0\nI1\ntp29\nRp30\n(I3\nS'|'\np31\n(g21\nI2\ntp32\nNNI2\nI1\nI0\ntp33\nbI4\ntp34\nsI6\nI1\nI0\ntp35\nbI00\nS'\\x01\\x01\\x01\\x01\\x01\\x02'\np36\ntp37\nb."
+        a = np.array([(1,(1,2))], dtype=[('a', 'i1', (2,2)), ('b', 'i1', 2)])
+        p = self._loads(asbytes(s))
+        assert_equal(a, p)
+
 
 class TestFancyIndexing(TestCase):
     def test_list(self):
@@ -690,6 +949,37 @@ class TestArgmax(TestCase):
         ([complex(0, 0), complex(0, 2), complex(0, 1)], 1),
         ([complex(1, 0), complex(0, 2), complex(0, 1)], 0),
         ([complex(1, 0), complex(0, 2), complex(1, 1)], 2),
+
+        ([np.datetime64('1923-04-14T12:43:12'),
+          np.datetime64('1994-06-21T14:43:15'),
+          np.datetime64('2001-10-15T04:10:32'),
+          np.datetime64('1995-11-25T16:02:16'),
+          np.datetime64('2005-01-04T03:14:12'),
+          np.datetime64('2041-12-03T14:05:03')], 5),
+        ([np.datetime64('1935-09-14T04:40:11'),
+          np.datetime64('1949-10-12T12:32:11'),
+          np.datetime64('2010-01-03T05:14:12'),
+          np.datetime64('2015-11-20T12:20:59'),
+          np.datetime64('1932-09-23T10:10:13'),
+          np.datetime64('2014-10-10T03:50:30')], 3),
+        ([np.datetime64('2059-03-14T12:43:12'),
+          np.datetime64('1996-09-21T14:43:15'),
+          np.datetime64('2001-10-15T04:10:32'),
+          np.datetime64('2022-12-25T16:02:16'),
+          np.datetime64('1963-10-04T03:14:12'),
+          np.datetime64('2013-05-08T18:15:23')], 0),
+
+        ([timedelta(days=5, seconds=14), timedelta(days=2, seconds=35),
+          timedelta(days=-1, seconds=23)], 0),
+        ([timedelta(days=1, seconds=43), timedelta(days=10, seconds=5),
+          timedelta(days=5, seconds=14)], 1),
+        ([timedelta(days=10, seconds=24), timedelta(days=10, seconds=5),
+          timedelta(days=10, seconds=43)], 2),
+
+        # Can't reduce a "flexible type"
+        #(['a', 'z', 'aa', 'zz'], 3),
+        #(['zz', 'a', 'aa', 'a'], 0),
+        #(['aa', 'z', 'zz', 'a'], 2),
     ]
 
     def test_all(self):
@@ -702,13 +992,78 @@ class TestArgmax(TestCase):
             assert all(amax == aargmax.choose(*a.transpose(i,*axes)))
 
     def test_combinations(self):
-        err = np.seterr(invalid="ignore")
-        try:
-            for arr, pos in self.nan_arr:
-                assert_equal(np.argmax(arr), pos, err_msg="%r"%arr)
-                assert_equal(arr[np.argmax(arr)], np.max(arr), err_msg="%r"%arr)
-        finally:
-            np.seterr(**err)
+        for arr, pos in self.nan_arr:
+            assert_equal(np.argmax(arr), pos, err_msg="%r"%arr)
+            assert_equal(arr[np.argmax(arr)], np.max(arr), err_msg="%r"%arr)
+
+
+class TestArgmin(TestCase):
+
+    nan_arr = [
+        ([0, 1, 2, 3, np.nan], 4),
+        ([0, 1, 2, np.nan, 3], 3),
+        ([np.nan, 0, 1, 2, 3], 0),
+        ([np.nan, 0, np.nan, 2, 3], 0),
+        ([0, 1, 2, 3, complex(0,np.nan)], 4),
+        ([0, 1, 2, 3, complex(np.nan,0)], 4),
+        ([0, 1, 2, complex(np.nan,0), 3], 3),
+        ([0, 1, 2, complex(0,np.nan), 3], 3),
+        ([complex(0,np.nan), 0, 1, 2, 3], 0),
+        ([complex(np.nan, np.nan), 0, 1, 2, 3], 0),
+        ([complex(np.nan, 0), complex(np.nan, 2), complex(np.nan, 1)], 0),
+        ([complex(np.nan, np.nan), complex(np.nan, 2), complex(np.nan, 1)], 0),
+        ([complex(np.nan, 0), complex(np.nan, 2), complex(np.nan, np.nan)], 0),
+
+        ([complex(0, 0), complex(0, 2), complex(0, 1)], 0),
+        ([complex(1, 0), complex(0, 2), complex(0, 1)], 2),
+        ([complex(1, 0), complex(0, 2), complex(1, 1)], 1),
+
+        ([np.datetime64('1923-04-14T12:43:12'),
+          np.datetime64('1994-06-21T14:43:15'),
+          np.datetime64('2001-10-15T04:10:32'),
+          np.datetime64('1995-11-25T16:02:16'),
+          np.datetime64('2005-01-04T03:14:12'),
+          np.datetime64('2041-12-03T14:05:03')], 0),
+        ([np.datetime64('1935-09-14T04:40:11'),
+          np.datetime64('1949-10-12T12:32:11'),
+          np.datetime64('2010-01-03T05:14:12'),
+          np.datetime64('2014-11-20T12:20:59'),
+          np.datetime64('2015-09-23T10:10:13'),
+          np.datetime64('1932-10-10T03:50:30')], 5),
+        ([np.datetime64('2059-03-14T12:43:12'),
+          np.datetime64('1996-09-21T14:43:15'),
+          np.datetime64('2001-10-15T04:10:32'),
+          np.datetime64('2022-12-25T16:02:16'),
+          np.datetime64('1963-10-04T03:14:12'),
+          np.datetime64('2013-05-08T18:15:23')], 4),
+
+        ([timedelta(days=5, seconds=14), timedelta(days=2, seconds=35),
+          timedelta(days=-1, seconds=23)], 2),
+        ([timedelta(days=1, seconds=43), timedelta(days=10, seconds=5),
+          timedelta(days=5, seconds=14)], 0),
+        ([timedelta(days=10, seconds=24), timedelta(days=10, seconds=5),
+          timedelta(days=10, seconds=43)], 1),
+
+        # Can't reduce a "flexible type"
+        #(['a', 'z', 'aa', 'zz'], 0),
+        #(['zz', 'a', 'aa', 'a'], 1),
+        #(['aa', 'z', 'zz', 'a'], 3),
+    ]
+
+    def test_all(self):
+        a = np.random.normal(0,1,(4,5,6,7,8))
+        for i in xrange(a.ndim):
+            amin = a.min(i)
+            aargmin = a.argmin(i)
+            axes = range(a.ndim)
+            axes.remove(i)
+            assert_(all(amin == aargmin.choose(*a.transpose(i,*axes))))
+
+    def test_combinations(self):
+        for arr, pos in self.nan_arr:
+            assert_equal(np.argmin(arr), pos, err_msg="%r"%arr)
+            assert_equal(arr[np.argmin(arr)], np.min(arr), err_msg="%r"%arr)
+
 
 
 class TestMinMax(TestCase):
@@ -793,7 +1148,7 @@ class TestClip(TestCase):
         assert np.all(x <= 4)
 
 
-class TestPutmask(TestCase):
+class TestPutmask:
     def tst_basic(self,x,T,mask,val):
         np.putmask(x,mask,val)
         assert np.all(x[mask] == T(val))
@@ -812,8 +1167,7 @@ class TestPutmask(TestCase):
                         yield self.tst_basic,x.copy().astype(T),T,mask,val
 
     def test_mask_size(self):
-        self.assertRaises(ValueError, np.putmask,
-                              np.array([1,2,3]), [True], 5)
+        assert_raises(ValueError, np.putmask, np.array([1,2,3]), [True], 5)
 
     def tst_byteorder(self,dtype):
         x = np.array([1,2,3],dtype)
@@ -840,7 +1194,7 @@ class TestPutmask(TestCase):
         pass
 
 
-class TestTake(TestCase):
+class TestTake:
     def tst_basic(self,x):
         ind = range(x.shape[0])
         assert_array_equal(x.take(ind, axis=0), x)
@@ -858,8 +1212,8 @@ class TestTake(TestCase):
     def test_raise(self):
         x = np.random.random(24)*100
         x.shape = 2,3,4
-        self.assertRaises(IndexError, x.take, [0,1,2], axis=0)
-        self.assertRaises(IndexError, x.take, [-3], axis=0)
+        assert_raises(IndexError, x.take, [0,1,2], axis=0)
+        assert_raises(IndexError, x.take, [-3], axis=0)
         assert_array_equal(x.take([-1], axis=0)[0], x[1])
 
     def test_clip(self):
@@ -922,6 +1276,11 @@ class TestIO(object):
             os.unlink(self.filename)
             #tmp_file.close()
 
+    def test_bool_fromstring(self):
+        v = np.array([True,False,True,False], dtype=np.bool_)
+        y = np.fromstring('1 0 -2.3 0.0', sep=' ', dtype=np.bool_)
+        assert_array_equal(v, y)
+
     def test_empty_files_binary(self):
         f = open(self.filename, 'w')
         f.close()
@@ -1002,6 +1361,31 @@ class TestIO(object):
                          array([1,2,3,4]),
                          dtype='<f4')
 
+    @dec.slow # takes > 1 minute on mechanical hard drive
+    def test_big_binary(self):
+        """Test workarounds for 32-bit limited fwrite, fseek, and ftell
+        calls in windows. These normally would hang doing something like this.
+        See http://projects.scipy.org/numpy/ticket/1660"""
+        if sys.platform != 'win32':
+            return
+        try:
+            # before workarounds, only up to 2**32-1 worked
+            fourgbplus = 2**32 + 2**16
+            testbytes = np.arange(8, dtype=np.int8)
+            n = len(testbytes)
+            flike = tempfile.NamedTemporaryFile()
+            f = flike.file
+            np.tile(testbytes, fourgbplus // testbytes.nbytes).tofile(f)
+            flike.seek(0)
+            a = np.fromfile(f, dtype=np.int8)
+            flike.close()
+            assert_(len(a) == fourgbplus)
+            # check only start and end for speed:
+            assert_((a[:n] == testbytes).all())
+            assert_((a[-n:] == testbytes).all())
+        except (MemoryError, ValueError):
+            pass
+
     def test_string(self):
         self._check_from('1,2,3,4', [1., 2., 3., 4.], sep=',')
 
@@ -1031,6 +1415,17 @@ class TestIO(object):
         v = np.array([1,2,3,4], dtype=np.int_)
         self._check_from('1,2,3,4', v, sep=',', dtype=np.int_)
 
+    def test_dtype_bool(self):
+        # can't use _check_from because fromstring can't handle True/False
+        v = np.array([True, False, True, False], dtype=np.bool_)
+        s = '1,0,-2.3,0'
+        f = open(self.filename, 'wb')
+        f.write(asbytes(s))
+        f.close()
+        y = np.fromfile(self.filename, sep=',', dtype=np.bool_)
+        assert_(y.dtype == '?')
+        assert_array_equal(y, v)
+
     def test_tofile_sep(self):
         x = np.array([1.51, 2, 3.51, 4], dtype=float)
         f = open(self.filename, 'w')
@@ -1063,7 +1458,7 @@ class TestIO(object):
         in_foreign_locale(self.test_tofile_format)()
 
 
-class TestFromBuffer(TestCase):
+class TestFromBuffer:
     def tst_basic(self,buffer,expected,kwargs):
         assert_array_equal(np.frombuffer(buffer,**kwargs),expected)
 
@@ -1075,6 +1470,9 @@ class TestFromBuffer(TestCase):
                 buf = x.tostring()
                 yield self.tst_basic,buf,x.flat,{'dtype':dt}
 
+    def test_empty(self):
+        yield self.tst_basic, asbytes(''), np.array([]), {}
+
 
 class TestResize(TestCase):
     def test_basic(self):
@@ -1188,6 +1586,68 @@ class TestStats(TestCase):
         res = dat.var(1)
         assert res.info == dat.info
 
+class TestDot(TestCase):
+    def test_dot_2args(self):
+        from numpy.core.multiarray import dot
+
+        a = np.array([[1, 2], [3, 4]], dtype=float)
+        b = np.array([[1, 0], [1, 1]], dtype=float)
+        c = np.array([[3, 2], [7, 4]], dtype=float)
+
+        d = dot(a, b)
+        assert_allclose(c, d)
+
+    def test_dot_3args(self):
+        from numpy.core.multiarray import dot
+
+        np.random.seed(22)
+        f = np.random.random_sample((1024, 16))
+        v = np.random.random_sample((16, 32))
+
+        r = np.empty((1024, 32))
+        for i in xrange(12):
+            dot(f,v,r)
+        assert_equal(sys.getrefcount(r), 2)
+        r2 = dot(f,v,out=None)
+        assert_array_equal(r2, r)
+        assert_(r is dot(f,v,out=r))
+
+        v = v[:,0].copy() # v.shape == (16,)
+        r = r[:,0].copy() # r.shape == (1024,)
+        r2 = dot(f,v)
+        assert_(r is dot(f,v,r))
+        assert_array_equal(r2, r)
+
+    def test_dot_3args_errors(self):
+        from numpy.core.multiarray import dot
+
+        np.random.seed(22)
+        f = np.random.random_sample((1024, 16))
+        v = np.random.random_sample((16, 32))
+
+        r = np.empty((1024, 31))
+        assert_raises(ValueError, dot, f, v, r)
+
+        r = np.empty((1024,))
+        assert_raises(ValueError, dot, f, v, r)
+
+        r = np.empty((32,))
+        assert_raises(ValueError, dot, f, v, r)
+
+        r = np.empty((32, 1024))
+        assert_raises(ValueError, dot, f, v, r)
+        assert_raises(ValueError, dot, f, v, r.T)
+
+        r = np.empty((1024, 64))
+        assert_raises(ValueError, dot, f, v, r[:,::2])
+        assert_raises(ValueError, dot, f, v, r[:,:32])
+
+        r = np.empty((1024, 32), dtype=np.float32)
+        assert_raises(ValueError, dot, f, v, r)
+
+        r = np.empty((1024, 32), dtype=int)
+        assert_raises(ValueError, dot, f, v, r)
+
 
 class TestSummarization(TestCase):
     def test_1d(self):
@@ -1243,23 +1703,23 @@ class TestNeighborhoodIter(TestCase):
     def _test_simple2d(self, dt):
         # Test zero and one padding for simple data type
         x = np.array([[0, 1], [2, 3]], dtype=dt)
-        r = [np.array([[0, 0, 0], [0, 0, 1]], dtype=dt), 
-             np.array([[0, 0, 0], [0, 1, 0]], dtype=dt), 
-             np.array([[0, 0, 1], [0, 2, 3]], dtype=dt), 
+        r = [np.array([[0, 0, 0], [0, 0, 1]], dtype=dt),
+             np.array([[0, 0, 0], [0, 1, 0]], dtype=dt),
+             np.array([[0, 0, 1], [0, 2, 3]], dtype=dt),
              np.array([[0, 1, 0], [2, 3, 0]], dtype=dt)]
         l = test_neighborhood_iterator(x, [-1, 0, -1, 1], x[0], NEIGH_MODE['zero'])
         assert_array_equal(l, r)
 
-        r = [np.array([[1, 1, 1], [1, 0, 1]], dtype=dt), 
-             np.array([[1, 1, 1], [0, 1, 1]], dtype=dt), 
-             np.array([[1, 0, 1], [1, 2, 3]], dtype=dt), 
+        r = [np.array([[1, 1, 1], [1, 0, 1]], dtype=dt),
+             np.array([[1, 1, 1], [0, 1, 1]], dtype=dt),
+             np.array([[1, 0, 1], [1, 2, 3]], dtype=dt),
              np.array([[0, 1, 1], [2, 3, 1]], dtype=dt)]
         l = test_neighborhood_iterator(x, [-1, 0, -1, 1], x[0], NEIGH_MODE['one'])
         assert_array_equal(l, r)
 
-        r = [np.array([[4, 4, 4], [4, 0, 1]], dtype=dt), 
-             np.array([[4, 4, 4], [0, 1, 4]], dtype=dt), 
-             np.array([[4, 0, 1], [4, 2, 3]], dtype=dt), 
+        r = [np.array([[4, 4, 4], [4, 0, 1]], dtype=dt),
+             np.array([[4, 4, 4], [0, 1, 4]], dtype=dt),
+             np.array([[4, 0, 1], [4, 2, 3]], dtype=dt),
              np.array([[0, 1, 4], [2, 3, 4]], dtype=dt)]
         l = test_neighborhood_iterator(x, [-1, 0, -1, 1], 4, NEIGH_MODE['constant'])
         assert_array_equal(l, r)
@@ -1276,9 +1736,9 @@ class TestNeighborhoodIter(TestCase):
 
     def _test_mirror2d(self, dt):
         x = np.array([[0, 1], [2, 3]], dtype=dt)
-        r = [np.array([[0, 0, 1], [0, 0, 1]], dtype=dt), 
-             np.array([[0, 1, 1], [0, 1, 1]], dtype=dt), 
-             np.array([[0, 0, 1], [2, 2, 3]], dtype=dt), 
+        r = [np.array([[0, 0, 1], [0, 0, 1]], dtype=dt),
+             np.array([[0, 1, 1], [0, 1, 1]], dtype=dt),
+             np.array([[0, 0, 1], [2, 2, 3]], dtype=dt),
              np.array([[0, 1, 1], [2, 3, 3]], dtype=dt)]
         l = test_neighborhood_iterator(x, [-1, 0, -1, 1], x[0], NEIGH_MODE['mirror'])
         assert_array_equal(l, r)
@@ -1516,6 +1976,7 @@ class TestWarnings(object):
 
         warnings.simplefilter("error", np.ComplexWarning)
         assert_raises(np.ComplexWarning, x.__setitem__, slice(None), y)
+        assert_equal(x, [1,2])
         warnings.simplefilter("default", np.ComplexWarning)
 
 if sys.version_info >= (2, 6):
@@ -1572,6 +2033,17 @@ if sys.version_info >= (2, 6):
             self._check('^ixxxx', [('f0', 'i'), ('', 'V4')])
             self._check('^i7x', [('f0', 'i'), ('', 'V7')])
 
+        def test_native_padding_3(self):
+            dt = np.dtype([('a', 'b'), ('b', 'i'), ('sub', np.dtype('b,i')), ('c', 'i')], align=True)
+            self._check("T{b:a:xxxi:b:T{b:f0:=i:f1:}:sub:xxxi:c:}", dt)
+
+            dt = np.dtype([('a', 'b'), ('b', 'i'), ('c', 'b'), ('d', 'b'), ('e', 'b'), ('sub', np.dtype('b,i', align=True))])
+            self._check("T{b:a:=i:b:b:c:b:d:b:e:T{b:f0:xxxi:f1:}:sub:}", dt)
+
+        def test_padding_with_array_inside_struct(self):
+            dt = np.dtype([('a', 'b'), ('b', 'i'), ('c', 'b', (3,)), ('d', 'i')], align=True)
+            self._check("T{b:a:xxxi:b:3b:c:xi:d:}", dt)
+
         def test_byteorder_inside_struct(self):
             # The byte order after @T{=i} should be '=', not '@'.
             # Check this by noting the absence of native alignment.
@@ -1628,9 +2100,11 @@ if sys.version_info >= (2, 6):
                   ('l', 'S4'),
                   ('m', 'U4'),
                   ('n', 'V3'),
-                  ('o', '?')]
+                  ('o', '?'),
+                  ('p', np.half),
+                 ]
             x = np.array([(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-                           asbytes('aaaa'), 'bbbb', asbytes('xxx'), True)],
+                           asbytes('aaaa'), 'bbbb', asbytes('xxx'), True, 1.0)],
                          dtype=dt)
             self._check_roundtrip(x)
 
@@ -1662,6 +2136,25 @@ if sys.version_info >= (2, 6):
                 x = np.array([1,2,3], dtype='<q')
                 assert_raises(ValueError, self._check_roundtrip, x)
 
+        def test_roundtrip_half(self):
+            half_list = [
+                1.0,
+                -2.0,
+                6.5504 * 10**4, #  (max half precision)
+                2**-14, # ~= 6.10352 * 10**-5 (minimum positive normal)
+                2**-24, # ~= 5.96046 * 10**-8 (minimum strictly positive subnormal)
+                0.0,
+                -0.0,
+                float('+inf'),
+                float('-inf'),
+                0.333251953125, # ~= 1/3
+            ]
+
+            x = np.array(half_list, dtype='>e')
+            self._check_roundtrip(x)
+            x = np.array(half_list, dtype='<e')
+            self._check_roundtrip(x)
+
         def test_export_simple_1d(self):
             x = np.array([1,2,3,4,5], dtype='i')
             y = memoryview(x)
@@ -1712,9 +2205,11 @@ if sys.version_info >= (2, 6):
                   ('l', 'S4'),
                   ('m', 'U4'),
                   ('n', 'V3'),
-                  ('o', '?')]
+                  ('o', '?'),
+                  ('p', np.half),
+                 ]
             x = np.array([(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-                           asbytes('aaaa'), 'bbbb', asbytes('   '), True)],
+                           asbytes('aaaa'), 'bbbb', asbytes('   '), True, 1.0)],
                          dtype=dt)
             y = memoryview(x)
             assert_equal(y.shape, (1,))
@@ -1723,9 +2218,9 @@ if sys.version_info >= (2, 6):
 
             sz = sum([dtype(b).itemsize for a, b in dt])
             if dtype('l').itemsize == 4:
-                assert_equal(y.format, 'T{b:a:=h:b:i:c:l:d:^q:dx:B:e:@H:f:=I:g:L:h:^Q:hx:=f:i:d:j:^g:k:=Zf:ix:Zd:jx:^Zg:kx:4s:l:=4w:m:3x:n:?:o:}')
+                assert_equal(y.format, 'T{b:a:=h:b:i:c:l:d:^q:dx:B:e:@H:f:=I:g:L:h:^Q:hx:=f:i:d:j:^g:k:=Zf:ix:Zd:jx:^Zg:kx:4s:l:=4w:m:3x:n:?:o:@e:p:}')
             else:
-                assert_equal(y.format, 'T{b:a:=h:b:i:c:q:d:^q:dx:B:e:@H:f:=I:g:Q:h:^Q:hx:=f:i:d:j:^g:k:=Zf:ix:Zd:jx:^Zg:kx:4s:l:=4w:m:3x:n:?:o:}')
+                assert_equal(y.format, 'T{b:a:=h:b:i:c:q:d:^q:dx:B:e:@H:f:=I:g:Q:h:^Q:hx:=f:i:d:j:^g:k:=Zf:ix:Zd:jx:^Zg:kx:4s:l:=4w:m:3x:n:?:o:@e:p:}')
             assert_equal(y.strides, (sz,))
             assert_equal(y.itemsize, sz)
 
@@ -1767,6 +2262,29 @@ if sys.version_info >= (2, 6):
             count_2 = sys.getrefcount(np.core._internal)
             assert_equal(count_1, count_2)
 
+        def test_padded_struct_array(self):
+            dt1 = np.dtype([('a', 'b'), ('b', 'i'), ('sub', np.dtype('b,i')), ('c', 'i')], align=True)
+            x1 = np.arange(dt1.itemsize, dtype=np.int8).view(dt1)
+            self._check_roundtrip(x1)
+
+            dt2 = np.dtype([('a', 'b'), ('b', 'i'), ('c', 'b', (3,)), ('d', 'i')], align=True)
+            x2 = np.arange(dt2.itemsize, dtype=np.int8).view(dt2)
+            self._check_roundtrip(x2)
+
+            dt3 = np.dtype([('a', 'b'), ('b', 'i'), ('c', 'b'), ('d', 'b'), ('e', 'b'), ('sub', np.dtype('b,i', align=True))])
+            x3 = np.arange(dt3.itemsize, dtype=np.int8).view(dt3)
+            self._check_roundtrip(x3)
+
+
+    class TestArrayAttributeDeletion(object):
+        """ticket #2046, should not seqfault, raise AttributeError"""
+
+        def test_attribute_deletion(self):
+            a = np.ones(2)
+            attr =  ['shape', 'strides', 'data', 'dtype', 'real', 'imag', 'flat']
+            for s in attr:
+                assert_raises(AttributeError, delattr, a, s)
+
 
 if __name__ == "__main__":
     run_module_suite()
