@@ -18,16 +18,22 @@ class TestTypes(TestCase):
     def test_type_add(self, level=1):
         # list of types
         for k, atype in enumerate(types):
-            vala = atype(3)
-            val1 = np.array([3],dtype=atype)
+            a_scalar = atype(3)
+            a_array = np.array([3],dtype=atype)
             for l, btype in enumerate(types):
-                valb = btype(1)
-                val2 = np.array([1],dtype=btype)
-                val = vala+valb
-                valo = val1 + val2
-                assert val.dtype.num == valo.dtype.num and \
-                       val.dtype.char == valo.dtype.char, \
-                       "error with (%d,%d)" % (k,l)
+                b_scalar = btype(1)
+                b_array = np.array([1],dtype=btype)
+                c_scalar = a_scalar + b_scalar
+                c_array = a_array + b_array
+                # It was comparing the type numbers, but the new ufunc
+                # function-finding mechanism finds the lowest function
+                # to which both inputs can be cast - which produces 'l'
+                # when you do 'q' + 'b'.  The old function finding mechanism
+                # skipped ahead based on the first argument, but that
+                # does not produce properly symmetric results...
+                assert_equal(c_scalar.dtype, c_array.dtype,
+                           "error with types (%d/'%c' + %d/'%c')" %
+                            (k,np.dtype(atype).char,l,np.dtype(btype).char))
 
     def test_type_create(self, level=1):
         for k, atype in enumerate(types):
@@ -54,6 +60,28 @@ class TestPower(TestCase):
                 assert_almost_equal(b, 6765201, err_msg=msg)
 
 
+class TestComplexDivision(TestCase):
+    def test_zero_division(self):
+        err = np.seterr(over="ignore", invalid="ignore", divide='ignore')
+        try:
+            for t in [np.complex64, np.complex128]:
+                a = t(0.0)
+                b = t(1.0)
+                assert_(np.isinf(b/a))
+                b = t(complex(np.inf, np.inf))
+                assert_(np.isinf(b/a))
+                b = t(complex(np.inf, np.nan))
+                assert_(np.isinf(b/a))
+                b = t(complex(np.nan, np.inf))
+                assert_(np.isinf(b/a))
+                b = t(complex(np.nan, np.nan))
+                assert_(np.isnan(b/a))
+                b = t(0.)
+                assert_(np.isnan(b/a))
+        finally:
+            np.seterr(**err)
+
+
 class TestConversion(TestCase):
     def test_int_from_long(self):
         l = [1e6, 1e12, 1e18, -1e6, -1e12, -1e18]
@@ -75,7 +103,7 @@ class TestConversion(TestCase):
 #            assert_equal( val, val2 )
 
 
-class TestRepr(TestCase):
+class TestRepr:
     def _test_type_repr(self, t):
         finfo=np.finfo(t)
         last_fraction_bit_idx = finfo.nexp + finfo.nmant
@@ -102,10 +130,10 @@ class TestRepr(TestCase):
                 assert_equal(val, val2)
 
     def test_float_repr(self):
-	# long double test cannot work, because eval goes through a python
-	# float
+        # long double test cannot work, because eval goes through a python
+        # float
         for t in [np.float32, np.float64]:
-	    yield test_float_repr, t
+            yield self._test_type_repr, t
 
 if __name__ == "__main__":
     run_module_suite()
