@@ -1,5 +1,4 @@
-__all__ = ['atleast_1d','atleast_2d','atleast_3d','vstack','hstack',
-           'column_stack','row_stack', 'dstack','array_split','split','hsplit',
+__all__ = ['column_stack','row_stack', 'dstack','array_split','split','hsplit',
            'vsplit','dsplit','apply_over_axes','expand_dims',
            'apply_along_axis', 'kron', 'tile', 'get_array_wrap']
 
@@ -7,22 +6,22 @@ import numpy.core.numeric as _nx
 from numpy.core.numeric import asarray, zeros, newaxis, outer, \
      concatenate, isscalar, array, asanyarray
 from numpy.core.fromnumeric import product, reshape
+from numpy.core import hstack, vstack, atleast_3d
 
 def apply_along_axis(func1d,axis,arr,*args):
     """
-    Apply function to 1-D slices along the given axis.
+    Apply a function to 1-D slices along the given axis.
 
-    Execute `func1d(arr[i],*args)` where `func1d` takes 1-D arrays, `arr` is
-    the input array, and `i` is an integer that varies in order to apply the
-    function along the given axis for each 1-D subarray in `arr`.
+    Execute `func1d(a, *args)` where `func1d` operates on 1-D arrays and `a`
+    is a 1-D slice of `arr` along `axis`.
 
     Parameters
     ----------
     func1d : function
-        This function should be able to take 1-D arrays. It is applied to 1-D
+        This function should accept 1-D arrays. It is applied to 1-D
         slices of `arr` along the specified axis.
     axis : integer
-        Axis along which `func1d` is applied.
+        Axis along which `arr` is sliced.
     arr : ndarray
         Input array.
     args : any
@@ -31,9 +30,10 @@ def apply_along_axis(func1d,axis,arr,*args):
     Returns
     -------
     outarr : ndarray
-        The output array. The shape of `outarr` depends on the return
-        value of `func1d`. If it returns arrays with the same shape as the
-        input arrays it receives, `outarr` has the same shape as `arr`.
+        The output array. The shape of `outarr` is identical to the shape of
+        `arr`, except along the `axis` dimension, where the length of `outarr`
+        is equal to the size of the return value of `func1d`.  If `func1d`
+        returns a scalar `outarr` will have one fewer dimensions than `arr`.
 
     See Also
     --------
@@ -46,9 +46,21 @@ def apply_along_axis(func1d,axis,arr,*args):
     ...     return (a[0] + a[-1]) * 0.5
     >>> b = np.array([[1,2,3], [4,5,6], [7,8,9]])
     >>> np.apply_along_axis(my_func, 0, b)
-    array([4., 5., 6.])
+    array([ 4.,  5.,  6.])
     >>> np.apply_along_axis(my_func, 1, b)
-    array([2., 5., 8.])
+    array([ 2.,  5.,  8.])
+
+    For a function that doesn't return a scalar, the number of dimensions in
+    `outarr` is the same as `arr`.
+
+    >>> def new_func(a):
+    ...     \"\"\"Divide elements of a by 2.\"\"\"
+    ...     return a * 0.5
+    >>> b = np.array([[1,2,3], [4,5,6], [7,8,9]])
+    >>> np.apply_along_axis(new_func, 0, b)
+    array([[ 0.5,  1. ,  1.5],
+           [ 2. ,  2.5,  3. ],
+           [ 3.5,  4. ,  4.5]])
 
     """
     arr = asarray(arr)
@@ -112,29 +124,28 @@ def apply_over_axes(func, a, axes):
     """
     Apply a function repeatedly over multiple axes.
 
-    `func` is called as `res = func(a, axis)`, with `axis` the first element
-    of `axes`.  The result `res` of the function call has to have
-    the same or one less dimension(s) as `a`. If `res` has one less dimension
-    than `a`, a dimension is then inserted before `axis`.
-    The call to `func` is then repeated for each axis in  `axes`,
+    `func` is called as `res = func(a, axis)`, where `axis` is the first
+    element of `axes`.  The result `res` of the function call must have
+    either the same dimensions as `a` or one less dimension.  If `res`
+    has one less dimension than `a`, a dimension is inserted before
+    `axis`.  The call to `func` is then repeated for each axis in `axes`,
     with `res` as the first argument.
 
     Parameters
     ----------
     func : function
-        This function should take two arguments, `func(a, axis)`.
-    arr : ndarray
+        This function must take two arguments, `func(a, axis)`.
+    a : array_like
         Input array.
     axes : array_like
-        Axes over which `func` has to be applied, the elements should be
-        integers.
+        Axes over which `func` is applied; the elements must be integers.
 
     Returns
     -------
     val : ndarray
-        The output array. The number of dimensions is the same as `a`,
-        the shape can be different, this depends on whether `func` changes
-        the shape of its output with respect to its input.
+        The output array.  The number of dimensions is the same as `a`,
+        but the shape can be different.  This depends on whether `func`
+        changes the shape of its output with respect to its input.
 
     See Also
     --------
@@ -148,7 +159,6 @@ def apply_over_axes(func, a, axes):
     array([[[ 0,  1,  2,  3],
             [ 4,  5,  6,  7],
             [ 8,  9, 10, 11]],
-    <BLANKLINE>
            [[12, 13, 14, 15],
             [16, 17, 18, 19],
             [20, 21, 22, 23]]])
@@ -238,253 +248,15 @@ def expand_dims(a, axis):
         axis = axis + len(shape) + 1
     return a.reshape(shape[:axis] + (1,) + shape[axis:])
 
-
-def atleast_1d(*arys):
-    """
-    Convert inputs to arrays with at least one dimension.
-
-    Scalar inputs are converted to 1-dimensional arrays, whilst
-    higher-dimensional inputs are preserved.
-
-    Parameters
-    ----------
-    array1, array2, ... : array_like
-        One or more input arrays.
-
-    Returns
-    -------
-    ret : ndarray
-        An array, or sequence of arrays, each with ``a.ndim >= 1``.
-        Copies are made only if necessary.
-
-    See Also
-    --------
-    atleast_2d, atleast_3d
-
-    Examples
-    --------
-    >>> np.atleast_1d(1.0)
-    array([ 1.])
-
-    >>> x = np.arange(9.0).reshape(3,3)
-    >>> np.atleast_1d(x)
-    array([[ 0.,  1.,  2.],
-           [ 3.,  4.,  5.],
-           [ 6.,  7.,  8.]])
-    >>> np.atleast_1d(x) is x
-    True
-
-    >>> np.atleast_1d(1, [3, 4])
-    [array([1]), array([3, 4])]
-
-    """
-    res = []
-    for ary in arys:
-        res.append(array(ary,copy=False,subok=True,ndmin=1))
-    if len(res) == 1:
-        return res[0]
-    else:
-        return res
-
-def atleast_2d(*arys):
-    """
-    View inputs as arrays with at least two dimensions.
-
-    Parameters
-    ----------
-    array1, array2, ... : array_like
-        One or more array-like sequences.  Non-array inputs are converted
-        to arrays.  Arrays that already have two or more dimensions are
-        preserved.
-
-    Returns
-    -------
-    res, res2, ... : ndarray
-        An array, or tuple of arrays, each with ``a.ndim >= 2``.
-        Copies are avoided where possible, and views with two or more
-        dimensions are returned.
-
-    See Also
-    --------
-    atleast_1d, atleast_3d
-
-    Examples
-    --------
-    >>> numpy.atleast_2d(3.0)
-    array([[ 3.]])
-
-    >>> x = numpy.arange(3.0)
-    >>> numpy.atleast_2d(x)
-    array([[ 0.,  1.,  2.]])
-    >>> numpy.atleast_2d(x).base is x
-    True
-
-    >>> np.atleast_2d(1, [1, 2], [[1, 2]])
-    [array([[1]]), array([[1, 2]]), array([[1, 2]])]
-
-    """
-    res = []
-    for ary in arys:
-        res.append(array(ary,copy=False,subok=True,ndmin=2))
-    if len(res) == 1:
-        return res[0]
-    else:
-        return res
-
-def atleast_3d(*arys):
-    """
-    View inputs as arrays with at least three dimensions.
-
-    Parameters
-    ----------
-    array1, array2, ... : array_like
-        One or more array-like sequences.  Non-array inputs are converted
-        to arrays. Arrays that already have three or more dimensions are
-        preserved.
-
-    Returns
-    -------
-    res1, res2, ... : ndarray
-        An array, or tuple of arrays, each with ``a.ndim >= 3``.
-        Copies are avoided where possible, and views with three or more
-        dimensions are returned.  For example, a one-dimensional array of
-        shape ``N`` becomes a view of shape ``(1, N, 1)``.  An ``(M, N)``
-        array becomes a view of shape ``(N, M, 1)``.
-
-    See Also
-    --------
-    numpy.atleast_1d, numpy.atleast_2d
-
-    Examples
-    --------
-    >>> numpy.atleast_3d(3.0)
-    array([[[ 3.]]])
-
-    >>> x = numpy.arange(3.0)
-    >>> numpy.atleast_3d(x).shape
-    (1, 3, 1)
-
-    >>> x = numpy.arange(12.0).reshape(4,3)
-    >>> numpy.atleast_3d(x).shape
-    (4, 3, 1)
-    >>> numpy.atleast_3d(x).base is x
-    True
-
-    >>> for arr in np.atleast_3d(1, [1, 2], [[1, 2]]): print arr, "\\n"
-    ...
-    [[[1]]]
-
-    [[[1]
-      [2]]]
-
-    [[[1]
-      [2]]]
-
-    """
-    res = []
-    for ary in arys:
-        ary = asarray(ary)
-        if len(ary.shape) == 0:
-            result = ary.reshape(1,1,1)
-        elif len(ary.shape) == 1:
-            result = ary[newaxis,:,newaxis]
-        elif len(ary.shape) == 2:
-            result = ary[:,:,newaxis]
-        else:
-            result = ary
-        res.append(result)
-    if len(res) == 1:
-        return res[0]
-    else:
-        return res
-
-
-def vstack(tup):
-    """
-    Stack arrays vertically.
-
-    `vstack` can be used to rebuild arrays divided by `vsplit`.
-
-    Parameters
-    ----------
-    tup : sequence of arrays
-        Tuple containing arrays to be stacked.  The arrays must have the same
-        shape along all but the first axis.
-
-    See Also
-    --------
-    array_split : Split an array into a list of multiple sub-arrays of
-                  near-equal size.
-    split : Split array into a list of multiple sub-arrays of equal size.
-    vsplit : Split array into a list of multiple sub-arrays vertically.
-    dsplit : Split array into a list of multiple sub-arrays along the 3rd axis
-             (depth).
-    concatenate : Join arrays together.
-    hstack : Stack arrays in sequence horizontally (column wise).
-    dstack : Stack arrays in sequence depth wise (along third dimension).
-
-    Examples
-    --------
-    >>> a = np.array([1, 2, 3])
-    >>> b = np.array([2, 3, 4])
-    >>> np.vstack((a,b))
-    array([[1, 2, 3],
-           [2, 3, 4]])
-    >>> a = np.array([[1], [2], [3]])
-    >>> b = np.array([[2], [3], [4]])
-    >>> np.vstack((a,b))
-    array([[1],
-           [2],
-           [3],
-           [2],
-           [3],
-           [4]])
-
-    """
-    return _nx.concatenate(map(atleast_2d,tup),0)
-
-def hstack(tup):
-    """
-    Stack arrays in sequence horizontally (column wise)
-
-    Take a sequence of arrays and stack them horizontally to make
-    a single array. hstack will rebuild arrays divided by hsplit.
-
-    Parameters
-    ----------
-    tup : sequence of ndarrays
-        All arrays must have the same shape along all but the second axis.
-
-    Returns
-    -------
-    stacked : ndarray
-        Ndarray formed by stacking the given arrays.
-
-    Examples
-    --------
-    >>> a = np.array((1,2,3))
-    >>> b = np.array((2,3,4))
-    >>> np.hstack((a,b))
-    array([1, 2, 3, 2, 3, 4])
-    >>> a = np.array([[1],[2],[3]])
-    >>> b = np.array([[2],[3],[4]])
-    >>> np.hstack((a,b))
-    array([[1, 2],
-           [2, 3],
-           [3, 4]])
-
-    """
-    return _nx.concatenate(map(atleast_1d,tup),1)
-
 row_stack = vstack
 
 def column_stack(tup):
     """
-    Stack 1-D arrays as columns into a 2-D array
+    Stack 1-D arrays as columns into a 2-D array.
 
     Take a sequence of 1-D arrays and stack them as columns
     to make a single 2-D array. 2-D arrays are stacked as-is,
-    just like with hstack.  1-D arrays are turned into 2-D columns
+    just like with `hstack`.  1-D arrays are turned into 2-D columns
     first.
 
     Parameters
@@ -492,6 +264,19 @@ def column_stack(tup):
     tup : sequence of 1-D or 2-D arrays.
         Arrays to stack. All of them must have the same first dimension.
 
+    Returns
+    -------
+    stacked : 2-D array
+        The array formed by stacking the given arrays.
+
+    See Also
+    --------
+    hstack, vstack, concatenate
+
+    Notes
+    -----
+    This function is equivalent to ``np.vstack(tup).T``.
+
     Examples
     --------
     >>> a = np.array((1,2,3))
@@ -512,11 +297,12 @@ def column_stack(tup):
 
 def dstack(tup):
     """
-    Stack arrays in sequence depth wise (along third dimension)
+    Stack arrays in sequence depth wise (along third axis).
 
-    Take a sequence of arrays and stack them along the third axis.
+    Takes a sequence of arrays and stack them along the third axis
+    to make a single array. Rebuilds arrays divided by `dsplit`.
     This is a simple way to stack 2D arrays (images) into a single
-    3D array for processing. dstack will rebuild arrays divided by dsplit.
+    3D array for processing.
 
     Parameters
     ----------
@@ -524,6 +310,22 @@ def dstack(tup):
         Arrays to stack. All of them must have the same shape along all
         but the third axis.
 
+    Returns
+    -------
+    stacked : ndarray
+        The array formed by stacking the given arrays.
+
+    See Also
+    --------
+    vstack : Stack along first axis.
+    hstack : Stack along second axis.
+    concatenate : Join arrays.
+    dsplit : Split array along third axis.
+
+    Notes
+    -----
+    Equivalent to ``np.concatenate(tup, axis=2)``.
+
     Examples
     --------
     >>> a = np.array((1,2,3))
@@ -532,13 +334,12 @@ def dstack(tup):
     array([[[1, 2],
             [2, 3],
             [3, 4]]])
+
     >>> a = np.array([[1],[2],[3]])
     >>> b = np.array([[2],[3],[4]])
     >>> np.dstack((a,b))
     array([[[1, 2]],
-    <BLANKLINE>
            [[2, 3]],
-    <BLANKLINE>
            [[3, 4]]])
 
     """
@@ -556,13 +357,14 @@ def array_split(ary,indices_or_sections,axis = 0):
     """
     Split an array into multiple sub-arrays of equal or near-equal size.
 
-    Please refer to the `numpy.split` documentation.  The only difference
-    between these functions is that `array_split` allows `indices_or_sections`
-    to be an integer that does *not* equally divide the axis.
+    Please refer to the ``split`` documentation.  The only difference
+    between these functions is that ``array_split`` allows
+    `indices_or_sections` to be an integer that does *not* equally
+    divide the axis.
 
     See Also
     --------
-    numpy.split : Split array into multiple sub-arrays.
+    split : Split array into multiple sub-arrays of equal size.
 
     Examples
     --------
@@ -610,14 +412,14 @@ def split(ary,indices_or_sections,axis=0):
     ----------
     ary : ndarray
         Array to be divided into sub-arrays.
-    indices_or_sections: integer or 1D array
+    indices_or_sections : int or 1-D array
         If `indices_or_sections` is an integer, N, the array will be divided
         into N equal arrays along `axis`.  If such a split is not possible,
         an error is raised.
 
-        If `indices_or_sections` is a 1D array of sorted integers, the entries
+        If `indices_or_sections` is a 1-D array of sorted integers, the entries
         indicate where along `axis` the array is split.  For example,
-        ``[2, 3]`` would, for ``axis = 0``, result in
+        ``[2, 3]`` would, for ``axis=0``, result in
 
           - ary[:2]
           - ary[2:3]
@@ -625,12 +427,12 @@ def split(ary,indices_or_sections,axis=0):
 
         If an index exceeds the dimension of the array along `axis`,
         an empty sub-array is returned correspondingly.
-    axis : integer, optional
-        The axis along which to split.  Default is 0.
+    axis : int, optional
+        The axis along which to split, default is 0.
 
     Returns
     -------
-    sub-arrays : list
+    sub-arrays : list of ndarrays
         A list of sub-arrays.
 
     Raises
@@ -660,7 +462,6 @@ def split(ary,indices_or_sections,axis=0):
 
     >>> x = np.arange(8.0)
     >>> np.split(x, [3, 5, 6, 10])
-    <BLANKLINE>
     [array([ 0.,  1.,  2.]),
      array([ 3.,  4.]),
      array([ 5.]),
@@ -679,20 +480,25 @@ def split(ary,indices_or_sections,axis=0):
 
 def hsplit(ary,indices_or_sections):
     """
-    Split array into multiple sub-arrays horizontally.
+    Split an array into multiple sub-arrays horizontally (column-wise).
 
-    Please refer to the `numpy.split` documentation.  `hsplit` is
-    equivalent to `numpy.split` with ``axis = 1``.
+    Please refer to the `split` documentation.  `hsplit` is equivalent
+    to `split` with ``axis=1``, the array is always split along the second
+    axis regardless of the array dimension.
 
     See Also
     --------
-    split : Split array into multiple sub-arrays.
+    split : Split an array into multiple sub-arrays of equal size.
 
     Examples
     --------
     >>> x = np.arange(16.0).reshape(4, 4)
+    >>> x
+    array([[  0.,   1.,   2.,   3.],
+           [  4.,   5.,   6.,   7.],
+           [  8.,   9.,  10.,  11.],
+           [ 12.,  13.,  14.,  15.]])
     >>> np.hsplit(x, 2)
-    <BLANKLINE>
     [array([[  0.,   1.],
            [  4.,   5.],
            [  8.,   9.],
@@ -701,9 +507,7 @@ def hsplit(ary,indices_or_sections):
            [  6.,   7.],
            [ 10.,  11.],
            [ 14.,  15.]])]
-
-    >>> np.hsplit(x, array([3, 6]))
-    <BLANKLINE>
+    >>> np.hsplit(x, np.array([3, 6]))
     [array([[  0.,   1.,   2.],
            [  4.,   5.,   6.],
            [  8.,   9.,  10.],
@@ -714,6 +518,20 @@ def hsplit(ary,indices_or_sections):
            [ 15.]]),
      array([], dtype=float64)]
 
+    With a higher dimensional array the split is still along the second axis.
+
+    >>> x = np.arange(8.0).reshape(2, 2, 2)
+    >>> x
+    array([[[ 0.,  1.],
+            [ 2.,  3.]],
+           [[ 4.,  5.],
+            [ 6.,  7.]]])
+    >>> np.hsplit(x, 2)
+    [array([[[ 0.,  1.]],
+           [[ 4.,  5.]]]),
+     array([[[ 2.,  3.]],
+           [[ 6.,  7.]]])]
+
     """
     if len(_nx.shape(ary)) == 0:
         raise ValueError, 'hsplit only works on arrays of 1 or more dimensions'
@@ -724,14 +542,49 @@ def hsplit(ary,indices_or_sections):
 
 def vsplit(ary,indices_or_sections):
     """
-    Split array into multiple sub-arrays vertically.
+    Split an array into multiple sub-arrays vertically (row-wise).
 
-    Please refer to the `numpy.split` documentation.
+    Please refer to the ``split`` documentation.  ``vsplit`` is equivalent
+    to ``split`` with `axis=0` (default), the array is always split along the
+    first axis regardless of the array dimension.
 
     See Also
     --------
-    numpy.split : The default behaviour of this function implements
-                  `vsplit`.
+    split : Split an array into multiple sub-arrays of equal size.
+
+    Examples
+    --------
+    >>> x = np.arange(16.0).reshape(4, 4)
+    >>> x
+    array([[  0.,   1.,   2.,   3.],
+           [  4.,   5.,   6.,   7.],
+           [  8.,   9.,  10.,  11.],
+           [ 12.,  13.,  14.,  15.]])
+    >>> np.vsplit(x, 2)
+    [array([[ 0.,  1.,  2.,  3.],
+           [ 4.,  5.,  6.,  7.]]),
+     array([[  8.,   9.,  10.,  11.],
+           [ 12.,  13.,  14.,  15.]])]
+    >>> np.vsplit(x, np.array([3, 6]))
+    [array([[  0.,   1.,   2.,   3.],
+           [  4.,   5.,   6.,   7.],
+           [  8.,   9.,  10.,  11.]]),
+     array([[ 12.,  13.,  14.,  15.]]),
+     array([], dtype=float64)]
+
+    With a higher dimensional array the split is still along the first axis.
+
+    >>> x = np.arange(8.0).reshape(2, 2, 2)
+    >>> x
+    array([[[ 0.,  1.],
+            [ 2.,  3.]],
+           [[ 4.,  5.],
+            [ 6.,  7.]]])
+    >>> np.vsplit(x, 2)
+    [array([[[ 0.,  1.],
+            [ 2.,  3.]]]),
+     array([[[ 4.,  5.],
+            [ 6.,  7.]]])]
 
     """
     if len(_nx.shape(ary)) < 2:
@@ -742,73 +595,38 @@ def dsplit(ary,indices_or_sections):
     """
     Split array into multiple sub-arrays along the 3rd axis (depth).
 
-    Parameters
-    ----------
-    ary : ndarray
-        An array, with at least 3 dimensions, to be divided into sub-arrays
-        depth-wise, or along the third axis.
-    indices_or_sections: integer or 1D array
-        If `indices_or_sections` is an integer, N, the array will be divided
-        into N equal arrays along `axis`. If an equal split is not possible,
-        a ValueError is raised.
-
-        if `indices_or_sections` is a 1D array of sorted integers representing
-        indices along `axis`, the array will be divided such that each index
-        marks the start of each sub-array. If an index exceeds the dimension of
-        the array along `axis`, and empty sub-array is returned for that index.
-    axis : integer, optional
-      the axis along which to split.  Default is 0.
-
-    Returns
-    -------
-    sub-arrays : list
-        A list of sub-arrays.
+    Please refer to the `split` documentation.  `dsplit` is equivalent
+    to `split` with ``axis=2``, the array is always split along the third
+    axis provided the array dimension is greater than or equal to 3.
 
     See Also
     --------
-    array_split : Split an array into a list of multiple sub-arrays
-                  of near-equal size.
-    split : Split array into a list of multiple sub-arrays of equal size.
-    hsplit : Split array into a list of multiple sub-arrays horizontally
-    vsplit : Split array into a list of multiple sub-arrays vertically
-    concatenate : Join arrays together.
-    hstack : Stack arrays in sequence horizontally (column wise)
-    vstack : Stack arrays in sequence vertically (row wise)
-    dstack : Stack arrays in sequence depth wise (along third dimension)
-
-    Notes
-    -----
-    `dsplit` requires that sub-arrays are of equal shape, whereas
-    `array_split` allows for sub-arrays to have nearly-equal shape.
-    Equivalent to `split` with `axis` = 2.
+    split : Split an array into multiple sub-arrays of equal size.
 
     Examples
     --------
     >>> x = np.arange(16.0).reshape(2, 2, 4)
+    >>> x
+    array([[[  0.,   1.,   2.,   3.],
+            [  4.,   5.,   6.,   7.]],
+           [[  8.,   9.,  10.,  11.],
+            [ 12.,  13.,  14.,  15.]]])
     >>> np.dsplit(x, 2)
-    <BLANKLINE>
     [array([[[  0.,   1.],
             [  4.,   5.]],
-    <BLANKLINE>
            [[  8.,   9.],
             [ 12.,  13.]]]),
      array([[[  2.,   3.],
             [  6.,   7.]],
-    <BLANKLINE>
            [[ 10.,  11.],
             [ 14.,  15.]]])]
-    <BLANKLINE>
-    >>> x = np.arange(16.0).reshape(2, 2, 4)
-    >>> np.dsplit(x, array([3, 6]))
-    <BLANKLINE>
+    >>> np.dsplit(x, np.array([3, 6]))
     [array([[[  0.,   1.,   2.],
             [  4.,   5.,   6.]],
-    <BLANKLINE>
            [[  8.,   9.,  10.],
             [ 12.,  13.,  14.]]]),
      array([[[  3.],
             [  7.]],
-    <BLANKLINE>
            [[ 11.],
             [ 15.]]]),
      array([], dtype=float64)]
@@ -818,6 +636,19 @@ def dsplit(ary,indices_or_sections):
         raise ValueError, 'vsplit only works on arrays of 3 or more dimensions'
     return split(ary,indices_or_sections,2)
 
+def get_array_prepare(*args):
+    """Find the wrapper for the array with the highest priority.
+
+    In case of ties, leftmost wins. If no wrapper is found, return None
+    """
+    wrappers = [(getattr(x, '__array_priority__', 0), -i,
+                 x.__array_prepare__) for i, x in enumerate(args)
+                                   if hasattr(x, '__array_prepare__')]
+    wrappers.sort()
+    if wrappers:
+        return wrappers[-1][-1]
+    return None
+
 def get_array_wrap(*args):
     """Find the wrapper for the array with the highest priority.
 
@@ -897,11 +728,10 @@ def kron(a,b):
     >>> J1 = (0,) + J             # extend to ndim=4
     >>> S1 = (1,) + b.shape
     >>> K = tuple(np.array(I) * np.array(S1) + np.array(J1))
-    >>> C[K] == A[I]*B[J]
+    >>> c[K] == a[I]*b[J]
     True
 
     """
-    wrapper = get_array_wrap(a, b)
     b = asanyarray(b)
     a = array(a,copy=False,subok=True,ndmin=b.ndim)
     ndb, nda = b.ndim, a.ndim
@@ -924,6 +754,10 @@ def kron(a,b):
     axis = nd-1
     for _ in xrange(nd):
         result = concatenate(result, axis=axis)
+    wrapper = get_array_prepare(a, b)
+    if wrapper is not None:
+        result = wrapper(result)
+    wrapper = get_array_wrap(a, b)
     if wrapper is not None:
         result = wrapper(result)
     return result
@@ -933,6 +767,19 @@ def tile(A, reps):
     """
     Construct an array by repeating A the number of times given by reps.
 
+    If `reps` has length ``d``, the result will have dimension of
+    ``max(d, A.ndim)``.
+
+    If ``A.ndim < d``, `A` is promoted to be d-dimensional by prepending new
+    axes. So a shape (3,) array is promoted to (1, 3) for 2-D replication,
+    or shape (1, 1, 3) for 3-D replication. If this is not the desired
+    behavior, promote `A` to d-dimensions manually before calling this
+    function.
+
+    If ``A.ndim > d``, `reps` is promoted to `A`.ndim by pre-pending 1's to it.
+    Thus for an `A` of shape (2, 3, 4, 5), a `reps` of (2, 2) is treated as
+    (1, 1, 2, 2).
+
     Parameters
     ----------
     A : array_like
@@ -943,24 +790,11 @@ def tile(A, reps):
     Returns
     -------
     c : ndarray
-        The output array.
+        The tiled output array.
 
     See Also
     --------
-    repeat
-
-    Notes
-    -----
-    If `reps` has length d, the result will have dimension of max(d, `A`.ndim).
-
-    If `A`.ndim < d, `A` is promoted to be d-dimensional by prepending new
-    axes. So a shape (3,) array is promoted to (1,3) for 2-D replication,
-    or shape (1,1,3) for 3-D replication. If this is not the desired behavior,
-    promote `A` to d-dimensions manually before calling this function.
-
-    If `A`.ndim > d, `reps` is promoted to `A`.ndim by pre-pending 1's to it.
-    Thus for an `A` of shape (2,3,4,5), a `reps` of (2,2) is treated as
-    (1,1,2,2).
+    repeat : Repeat elements of an array.
 
     Examples
     --------
@@ -972,7 +806,6 @@ def tile(A, reps):
            [0, 1, 2, 0, 1, 2]])
     >>> np.tile(a, (2, 1, 2))
     array([[[0, 1, 2, 0, 1, 2]],
-    <BLANKLINE>
            [[0, 1, 2, 0, 1, 2]]])
 
     >>> b = np.array([[1, 2], [3, 4]])
