@@ -6,7 +6,6 @@ from __future__ import division
 import numpy as np
 import numpy.polynomial.polynomial as poly
 from numpy.testing import *
-from exceptions import TypeError, ValueError
 
 def trim(x) :
     return poly.polytrim(x, tol=1e-6)
@@ -62,6 +61,14 @@ class TestArithmetic(TestCase) :
                 res = poly.polysub([0]*i + [1], [0]*j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
+    def test_polymulx(self):
+        assert_equal(poly.polymulx([0]), [0])
+        assert_equal(poly.polymulx([1]), [0, 1])
+        for i in range(1, 5):
+            ser = [0]*i + [1]
+            tgt = [0]*(i + 1) + [1]
+            assert_equal(poly.polymulx(ser), tgt)
+
     def test_polymul(self) :
         for i in range(5) :
             for j in range(5) :
@@ -97,7 +104,7 @@ class TestArithmetic(TestCase) :
             return x*(x**2 - 1)
 
         #check empty input
-        assert_equal(poly.polyval([], 1).size, 0)
+        assert_equal(poly.polyval([], [1]).size, 0)
 
         #check normal input)
         x = np.linspace(-1,1)
@@ -122,10 +129,15 @@ class TestCalculus(TestCase) :
 
     def test_polyint(self) :
         # check exceptions
+        assert_raises(ValueError, poly.polyint, [0], .5)
         assert_raises(ValueError, poly.polyint, [0], -1)
         assert_raises(ValueError, poly.polyint, [0], 1, [0,0])
-        assert_raises(ValueError, poly.polyint, [0], 1, lbnd=[0,0])
-        assert_raises(ValueError, poly.polyint, [0], 1, scl=[0,0])
+
+        # test integration of zero polynomial
+        for i in range(2, 5):
+            k = [0]*(i - 2) + [1]
+            res = poly.polyint([0], m=i, k=k)
+            assert_almost_equal(res, [0, 1])
 
         # check single integration with integration constant
         for i in range(5) :
@@ -192,6 +204,7 @@ class TestCalculus(TestCase) :
 
     def test_polyder(self) :
         # check exceptions
+        assert_raises(ValueError, poly.polyder, [0], .5)
         assert_raises(ValueError, poly.polyder, [0], -1)
 
         # check that zeroth deriviative does nothing
@@ -242,6 +255,7 @@ class TestMisc(TestCase) :
         for i in range(4) :
             coef = [0]*i + [1]
             assert_almost_equal(v[...,i], poly.polyval(x, coef))
+
         # check for 2d x
         x = np.array([[1,2],[3,4],[5,6]])
         v = poly.polyvander(x, 3)
@@ -253,6 +267,7 @@ class TestMisc(TestCase) :
     def test_polyfit(self) :
         def f(x) :
             return x*(x - 1)*(x - 2)
+
         # Test exceptions
         assert_raises(ValueError, poly.polyfit, [1],    [1],     -1)
         assert_raises(TypeError,  poly.polyfit, [[1]],  [1],      0)
@@ -260,22 +275,40 @@ class TestMisc(TestCase) :
         assert_raises(TypeError,  poly.polyfit, [1],    [[[1]]],  0)
         assert_raises(TypeError,  poly.polyfit, [1, 2], [1],      0)
         assert_raises(TypeError,  poly.polyfit, [1],    [1, 2],   0)
+        assert_raises(TypeError,  poly.polyfit, [1],    [1],   0, w=[[1]])
+        assert_raises(TypeError,  poly.polyfit, [1],    [1],   0, w=[1,1])
+
         # Test fit
         x = np.linspace(0,2)
         y = f(x)
-        coef = poly.polyfit(x, y, 3)
-        assert_equal(len(coef), 4)
-        assert_almost_equal(poly.polyval(x, coef), y)
-        coef = poly.polyfit(x, y, 4)
-        assert_equal(len(coef), 5)
-        assert_almost_equal(poly.polyval(x, coef), y)
-        coef2d = poly.polyfit(x, np.array([y,y]).T, 4)
-        assert_almost_equal(coef2d, np.array([coef,coef]).T)
+        #
+        coef3 = poly.polyfit(x, y, 3)
+        assert_equal(len(coef3), 4)
+        assert_almost_equal(poly.polyval(x, coef3), y)
+        #
+        coef4 = poly.polyfit(x, y, 4)
+        assert_equal(len(coef4), 5)
+        assert_almost_equal(poly.polyval(x, coef4), y)
+        #
+        coef2d = poly.polyfit(x, np.array([y,y]).T, 3)
+        assert_almost_equal(coef2d, np.array([coef3,coef3]).T)
+        # test weighting
+        w = np.zeros_like(x)
+        yw = y.copy()
+        w[1::2] = 1
+        yw[0::2] = 0
+        wcoef3 = poly.polyfit(x, yw, 3, w=w)
+        assert_almost_equal(wcoef3, coef3)
+        #
+        wcoef2d = poly.polyfit(x, np.array([yw,yw]).T, 3, w=w)
+        assert_almost_equal(wcoef2d, np.array([coef3,coef3]).T)
 
     def test_polytrim(self) :
         coef = [2, -1, 1, 0]
+
         # Test exceptions
         assert_raises(ValueError, poly.polytrim, coef, -1)
+
         # Test results
         assert_equal(poly.polytrim(coef), coef[:-1])
         assert_equal(poly.polytrim(coef, 1), coef[:-3])
@@ -364,6 +397,17 @@ class TestPolynomialClass(TestCase) :
         xx = 2*x - 1
         assert_almost_equal(self.p2(x), self.p1(xx))
 
+    def test_degree(self) :
+        assert_equal(self.p1.degree(), 2)
+
+    def test_cutdeg(self) :
+        assert_raises(ValueError, self.p1.cutdeg, .5)
+        assert_raises(ValueError, self.p1.cutdeg, -1)
+        assert_equal(len(self.p1.cutdeg(3)), 3)
+        assert_equal(len(self.p1.cutdeg(2)), 3)
+        assert_equal(len(self.p1.cutdeg(1)), 2)
+        assert_equal(len(self.p1.cutdeg(0)), 1)
+
     def test_convert(self) :
         x = np.linspace(-1,1)
         p = self.p1.convert(domain=[0,1])
@@ -381,6 +425,7 @@ class TestPolynomialClass(TestCase) :
         assert_equal(p.trim(1e-5).coef, coef[:1])
 
     def test_truncate(self) :
+        assert_raises(ValueError, self.p1.truncate, .5)
         assert_raises(ValueError, self.p1.truncate, 0)
         assert_equal(len(self.p1.truncate(4)), 3)
         assert_equal(len(self.p1.truncate(3)), 3)
@@ -394,6 +439,8 @@ class TestPolynomialClass(TestCase) :
     def test_integ(self) :
         p = self.p2.integ()
         assert_almost_equal(p.coef, poly.polyint([1,2,3], 1, 0, scl=.5))
+        p = self.p2.integ(lbnd=0)
+        assert_almost_equal(p(0), 0)
         p = self.p2.integ(1, 1)
         assert_almost_equal(p.coef, poly.polyint([1,2,3], 1, 1, scl=.5))
         p = self.p2.integ(2, [1, 2])
@@ -410,6 +457,13 @@ class TestPolynomialClass(TestCase) :
         tgt = [0, .5, 1]
         assert_almost_equal(res, tgt)
 
+    def test_linspace(self):
+        xdes = np.linspace(0, 1, 20)
+        ydes = self.p2(xdes)
+        xres, yres = self.p2.linspace(20)
+        assert_almost_equal(xres, xdes)
+        assert_almost_equal(yres, ydes)
+
     def test_fromroots(self) :
         roots = [0, .5, 1]
         p = poly.Polynomial.fromroots(roots, domain=[0, 1])
@@ -422,11 +476,25 @@ class TestPolynomialClass(TestCase) :
             return x*(x - 1)*(x - 2)
         x = np.linspace(0,3)
         y = f(x)
+
+        # test default value of domain
         p = poly.Polynomial.fit(x, y, 3)
-        assert_almost_equal(p(x), y)
+        assert_almost_equal(p.domain, [0,3])
+
+        # test that fit works in given domains
         p = poly.Polynomial.fit(x, y, 3, None)
         assert_almost_equal(p(x), y)
         assert_almost_equal(p.domain, [0,3])
+        p = poly.Polynomial.fit(x, y, 3, [])
+        assert_almost_equal(p(x), y)
+        assert_almost_equal(p.domain, [-1, 1])
+        # test that fit accepts weights.
+        w = np.zeros_like(x)
+        yw = y.copy()
+        w[1::2] = 1
+        yw[0::2] = 0
+        p = poly.Polynomial.fit(x, yw, 3, w=w)
+        assert_almost_equal(p(x), y)
 
     def test_identity(self) :
         x = np.linspace(0,3)
@@ -434,3 +502,7 @@ class TestPolynomialClass(TestCase) :
         assert_almost_equal(p(x), x)
         p = poly.Polynomial.identity([1,3])
         assert_almost_equal(p(x), x)
+#
+
+if __name__ == "__main__":
+    run_module_suite()
